/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.internal.marshall;

import com.db4o.foundation.*;
import com.db4o.internal.*;

/**
 * @exclude
 */
public class ObjectHeaderAttributes1 extends ObjectHeaderAttributes{
    
    private static final byte VERSION = (byte)1;
    
    private final int _fieldCount;
    
    private final BitMap4 _nullBitMap;
    
    private int _baseLength;
    
    private int _payLoadLength;
    
    
    public ObjectHeaderAttributes1(ObjectReference yo) {
        _fieldCount = yo.getYapClass().fieldCount();
        _nullBitMap = new BitMap4(_fieldCount);
        calculateLengths(yo);
    }
    
    public ObjectHeaderAttributes1(Buffer reader){
        _fieldCount = reader.readInt();
        _nullBitMap = reader.readBitMap(_fieldCount);
    }
    
    public void addBaseLength(int length){
        _baseLength += length;
    }
    
    public void addPayLoadLength(int length){
        _payLoadLength += length;
    }
    
    private void calculateLengths(ObjectReference yo) {
        _baseLength = headerLength() + nullBitMapLength();
        _payLoadLength = 0;
        ClassMetadata yc = yo.getYapClass();
        Transaction trans = yo.getTrans();
        Object obj = yo.getObject();
        calculateLengths(trans, yc, obj, 0);
        _baseLength = yo.getStream().alignToBlockSize(_baseLength);        
    }
    
    private void calculateLengths(Transaction trans, ClassMetadata yc, Object obj, int fieldIndex) {
        _baseLength += Const4.INT_LENGTH;
        if (yc.i_fields != null) {
            for (int i = 0; i < yc.i_fields.length; i++) {
                FieldMetadata yf = yc.i_fields[i];
                Object child = yf.getOrCreate(trans, obj);
                if( child == null && yf.canUseNullBitmap()){
                    _nullBitMap.setTrue(fieldIndex);
                }else{
                    yf.calculateLengths(trans, this, child);
                }
                fieldIndex ++;
            }
        }
        if (yc.i_ancestor == null) {
            return;
        }
        calculateLengths(trans, yc.i_ancestor, obj, fieldIndex);
    }

    private int headerLength(){
        return Const4.OBJECT_LENGTH 
            + Const4.ID_LENGTH  // YapClass ID 
            + 1; // Marshaller Version 
    }
    
    public boolean isNull(int fieldIndex){
        return _nullBitMap.isTrue(fieldIndex);
    }
    
    private int nullBitMapLength(){
        return Const4.INT_LENGTH + _nullBitMap.marshalledLength();
    }

    public int objectLength(){
        return _baseLength + _payLoadLength;
    }
    
    public void prepareIndexedPayLoadEntry(Transaction trans){
        _payLoadLength =  trans.stream().alignToBlockSize(_payLoadLength);
    }
    
    public void write(StatefulBuffer writer){
        writer.append(VERSION);
        writer.writeInt(_fieldCount);
        writer.writeBitMap(_nullBitMap);
        writer._payloadOffset = _baseLength;
    }

}
