/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.util.io.win32;

import java.io.File;
import java.io.IOException;

import com.db4o.io.IoAdapter;

/**
 * An IoAdapter implementation that uses JNI to talk directly with the WIN32 API.
 */
public class Win32IoAdapter extends IoAdapter {
	
	static {
		System.loadLibrary("Win32IoAdapter");
	}
	
	private long _handle;

	public Win32IoAdapter(String path, boolean lockFile, long initialLength) {
		_handle = openFile(path, lockFile, initialLength);
	}	

	public Win32IoAdapter() {
	}

	public void close() throws IOException {
		closeFile(getHandle());
		_handle = 0;
	}

	public void delete(String path) {
		new File(path).delete();
	}

    public boolean exists(String path){
        File existingFile = new File(path);
        return  existingFile.exists() && existingFile.length() > 0;
    }

	public long getLength() throws IOException {
		return getLength(getHandle());
	}

	public IoAdapter open(String path, boolean lockFile, long initialLength)
			throws IOException {
		return new Win32IoAdapter(path, lockFile, initialLength);
	}

	public int read(byte[] bytes, int length) throws IOException {
		return read(getHandle(), bytes, length);
	}

	public void seek(long pos) throws IOException {
		seek(getHandle(), pos);
	}
	
	public void sync() throws IOException {
		sync(getHandle());
	}
	
	public void write(byte[] bytes, int length) throws IOException {
		write(getHandle(), bytes, length);
	}
	
	public void copy(long oldAddress, long newAddress, int length)
			throws IOException {
		copy(getHandle(), oldAddress, newAddress, length);
	}

	private long getHandle() {
		if (0 == _handle) {
			throw new IllegalStateException("File is not open.");
		}
		return _handle;
	}
	
	private static native long openFile(String path, boolean lockFile, long initialLength);
	
	private static native void closeFile(long handle);
	
	private static native long getLength(long handle);
	
	private static native int read(long handle, byte[] bytes, int length);
	
	private static native void seek(long handle, long pos);
	
	private static native void sync(long handle);
	
	private static native void write(long handle, byte[] bytes, int lenght);
	
	private static native void copy(long handle, long oldAddress, long newAddress, int length);
}
